'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _timm = require('timm');

var _storyboard = require('storyboard');

var _kebabCase = require('kebab-case');

var _kebabCase2 = _interopRequireDefault(_kebabCase);

var _readSpecs = require('./utils/readSpecs');

var _constants = require('./utils/constants');

var _helpers = require('./utils/helpers');

var _removeInternalLinks2 = require('./utils/removeInternalLinks');

var _removeInternalLinks3 = _interopRequireDefault(_removeInternalLinks2);

var _writeSpecs = require('./utils/writeSpecs');

var _writeSpecs2 = _interopRequireDefault(_writeSpecs);

var _shell = require('./utils/shell');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var PASS_THROUGH_OPTS = ['dev', 'peer', 'optional', 'exact', 'tilde', 'ignoreEngines'];

var run = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(pkgName0, op, deps, opts) {
    var src, ignoreSrc, linkPattern, pkgName, allSpecs, _allSpecs$pkgName, pkgPath, specPath, prevSpecs, cmd, pkgNames, externalDeps, externalOperation, _removeInternalLinks, nextSpecs, removedPackagesByType, succeeded, _cmd, _readOneSpec, updatedSpecs, finalSpecs, internalDeps, internalOperation, _readOneSpec2, specs, _nextSpecs;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            src = opts.src, ignoreSrc = opts.ignoreSrc, linkPattern = opts.link;
            pkgName = pkgName0 === '.' || pkgName0 === 'ROOT' ? _readSpecs.ROOT_PACKAGE : pkgName0;
            _context.next = 4;
            return (0, _readSpecs.readAllSpecs)(src, ignoreSrc);

          case 4:
            allSpecs = _context.sent;

            if (!allSpecs[pkgName]) {
              _storyboard.mainStory.error('No such package: ' + pkgName);
              process.exit(1);
            }
            _allSpecs$pkgName = allSpecs[pkgName], pkgPath = _allSpecs$pkgName.pkgPath, specPath = _allSpecs$pkgName.specPath, prevSpecs = _allSpecs$pkgName.specs;

            // Very little to do when using yarn workspaces ;)

            if (!opts.workspaces) {
              _context.next = 13;
              break;
            }

            _storyboard.mainStory.info('Using yarn workspaces...');
            cmd = getYarnCommand(op, deps, opts);
            _context.next = 12;
            return (0, _shell.exec)(cmd, { cwd: pkgPath });

          case 12:
            return _context.abrupt('return');

          case 13:

            // Proceed, the old way
            pkgNames = Object.keys(allSpecs);

            // Add/remove/upgrade EXTERNAL dependencies:
            // 1. Remove internal links from package.json
            // 2. Run `yarn add/remove/upgrade` as needed (if it fails, revert to original specs and abort)
            // 3. Add the original internal links back to package.json

            externalDeps = deps.filter(function (dep) {
              return !isLinked(pkgNames, linkPattern, dep);
            });
            externalOperation = externalDeps.length || op === 'upgrade' && !deps.length;

            if (!externalOperation) {
              _context.next = 37;
              break;
            }

            _removeInternalLinks = (0, _removeInternalLinks3.default)(prevSpecs, pkgNames, linkPattern), nextSpecs = _removeInternalLinks.nextSpecs, removedPackagesByType = _removeInternalLinks.removedPackagesByType;
            succeeded = false;
            _context.prev = 19;

            if (nextSpecs !== prevSpecs) (0, _writeSpecs2.default)(specPath, nextSpecs);
            _storyboard.mainStory.info('Executing \'yarn ' + op + '\'...');
            _cmd = getYarnCommand(op, externalDeps, opts);
            _context.next = 25;
            return (0, _shell.exec)(_cmd, { cwd: pkgPath });

          case 25:
            succeeded = true;
            _context.next = 30;
            break;

          case 28:
            _context.prev = 28;
            _context.t0 = _context['catch'](19);

          case 30:
            if (succeeded) {
              _context.next = 33;
              break;
            }

            if (prevSpecs != null) (0, _writeSpecs2.default)(specPath, prevSpecs);
            return _context.abrupt('return');

          case 33:
            // Read the updated package.json, and add the internal deps
            _readOneSpec = (0, _readSpecs.readOneSpec)(pkgPath), updatedSpecs = _readOneSpec.specs;
            finalSpecs = updatedSpecs;

            Object.keys(removedPackagesByType).forEach(function (type) {
              var removedPackages = removedPackagesByType[type];
              var nextDeps = (0, _timm.merge)(updatedSpecs[type] || {}, removedPackages);
              finalSpecs = (0, _timm.set)(finalSpecs, type, nextDeps);
            });
            (0, _writeSpecs2.default)(specPath, finalSpecs);

          case 37:

            // Add/remove/upgrade INTERNAL dependencies:
            internalDeps = deps.filter(function (dep) {
              return isLinked(pkgNames, linkPattern, dep);
            });
            internalOperation = internalDeps.length || op === 'upgrade' && !deps.length;

            if (!internalOperation) {
              _context.next = 58;
              break;
            }

            _storyboard.mainStory.info('Processing \'' + op + '\' on internal dependencies...');
            _readOneSpec2 = (0, _readSpecs.readOneSpec)(pkgPath), specs = _readOneSpec2.specs;
            _nextSpecs = void 0;
            _context.t1 = op;
            _context.next = _context.t1 === 'add' ? 46 : _context.t1 === 'remove' ? 50 : _context.t1 === 'upgrade' ? 54 : 56;
            break;

          case 46:
            _context.next = 48;
            return addInternal(specs, internalDeps, pkgPath, allSpecs, opts);

          case 48:
            _nextSpecs = _context.sent;
            return _context.abrupt('break', 57);

          case 50:
            _context.next = 52;
            return removeInternal(specs, internalDeps, pkgPath);

          case 52:
            _nextSpecs = _context.sent;
            return _context.abrupt('break', 57);

          case 54:
            _nextSpecs = upgradeInternal(specs, internalDeps, allSpecs, linkPattern);
            return _context.abrupt('break', 57);

          case 56:
            throw new Error('INVALID_ADD_REMOVE_UPGRADE_COMMAND');

          case 57:
            if (_nextSpecs !== prevSpecs) (0, _writeSpecs2.default)(specPath, _nextSpecs);

          case 58:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[19, 28]]);
  }));

  return function run(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var addInternal = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(prevSpecs, deps, pkgPath, allSpecs, opts) {
    var nextSpecs, i, _parseDep, depName, depVersion0, depType, depVersion;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            nextSpecs = prevSpecs;
            i = 0;

          case 2:
            if (!(i < deps.length)) {
              _context2.next = 20;
              break;
            }

            _parseDep = (0, _helpers.parseDep)(deps[i]), depName = _parseDep.name, depVersion0 = _parseDep.version;
            _context2.prev = 4;

            _storyboard.mainStory.info('Linking ' + _storyboard.chalk.cyan.bold(depName) + '...');
            _context2.next = 8;
            return (0, _shell.exec)('yarn link ' + depName, {
              cwd: pkgPath,
              logLevel: 'trace',
              errorLogLevel: 'trace'
            });

          case 8:
            _context2.next = 12;
            break;

          case 10:
            _context2.prev = 10;
            _context2.t0 = _context2['catch'](4);

          case 12:
            depType = void 0;

            if (opts.dev) depType = 'devDependencies';else if (opts.peer) depType = 'peerDependencies';else if (opts.optional) depType = 'optionalDependencies';else depType = 'dependencies';
            depVersion = depVersion0;

            if (!depVersion) {
              depVersion = allSpecs[depName] ? allSpecs[depName].specs.version : '*';
              if (depVersion !== '*') {
                if (opts.tilde) depVersion = '~' + depVersion;else if (!opts.exact) depVersion = '^' + depVersion;
              }
            }
            nextSpecs = (0, _timm.setIn)(nextSpecs, [depType, depName], depVersion);

          case 17:
            i++;
            _context2.next = 2;
            break;

          case 20:
            return _context2.abrupt('return', nextSpecs);

          case 21:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined, [[4, 10]]);
  }));

  return function addInternal(_x5, _x6, _x7, _x8, _x9) {
    return _ref2.apply(this, arguments);
  };
}();

var removeInternal = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(prevSpecs, deps, pkgPath) {
    var nextSpecs, i, _parseDep2, depName, k, type;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            nextSpecs = prevSpecs;
            i = 0;

          case 2:
            if (!(i < deps.length)) {
              _context3.next = 24;
              break;
            }

            _parseDep2 = (0, _helpers.parseDep)(deps[i]), depName = _parseDep2.name;
            _context3.prev = 4;

            _storyboard.mainStory.info('Unlinking ' + _storyboard.chalk.cyan.bold(depName) + '...');
            _context3.next = 8;
            return (0, _shell.exec)('yarn unlink ' + depName, {
              cwd: pkgPath,
              logLevel: 'trace',
              errorLogLevel: 'trace'
            });

          case 8:
            _context3.next = 12;
            break;

          case 10:
            _context3.prev = 10;
            _context3.t0 = _context3['catch'](4);

          case 12:
            k = 0;

          case 13:
            if (!(k < _constants.DEP_TYPES.length)) {
              _context3.next = 21;
              break;
            }

            type = _constants.DEP_TYPES[k];

            if (nextSpecs[type]) {
              _context3.next = 17;
              break;
            }

            return _context3.abrupt('continue', 18);

          case 17:
            nextSpecs = (0, _timm.setIn)(nextSpecs, [type, depName], undefined);

          case 18:
            k++;
            _context3.next = 13;
            break;

          case 21:
            i++;
            _context3.next = 2;
            break;

          case 24:
            return _context3.abrupt('return', nextSpecs);

          case 25:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined, [[4, 10]]);
  }));

  return function removeInternal(_x10, _x11, _x12) {
    return _ref3.apply(this, arguments);
  };
}();

var upgradeInternal = function upgradeInternal(prevSpecs, deps, allSpecs, linkPattern) {
  var pkgNames = Object.keys(allSpecs);
  var nextSpecs = prevSpecs;
  var targetVersions = {};
  deps.forEach(function (dep) {
    var _parseDep3 = (0, _helpers.parseDep)(dep),
        name = _parseDep3.name,
        version = _parseDep3.version;

    targetVersions[name] = version;
  });
  _constants.DEP_TYPES.forEach(function (type) {
    Object.keys(nextSpecs[type] || {}).forEach(function (depName) {
      if (!isLinked(pkgNames, linkPattern, depName)) return;
      var depVersion = targetVersions[depName];
      if (!depVersion && allSpecs[depName]) {
        depVersion = '^' + allSpecs[depName].specs.version;
      }
      nextSpecs = (0, _timm.setIn)(nextSpecs, [type, depName], depVersion);
    });
  });
  return nextSpecs;
};

var isLinked = function isLinked(pkgNames, linkPattern, dep) {
  var _parseDep4 = (0, _helpers.parseDep)(dep),
      pkgName = _parseDep4.name;

  if (pkgNames.indexOf(pkgName) >= 0) return true;
  if (linkPattern && new RegExp(linkPattern).test(pkgName)) return true;
  return false;
};

var getYarnCommand = function getYarnCommand(op, dependencies, options) {
  var cmd = 'yarn ' + op;
  if (dependencies.length) cmd += ' ' + dependencies.join(' ');
  PASS_THROUGH_OPTS.forEach(function (key) {
    if (options[key]) cmd += ' --' + (0, _kebabCase2.default)(key);
  });
  return cmd;
};

exports.default = run;