#!/usr/bin/env node
'use strict';

require('babel-polyfill');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _timm = require('timm');

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _initConsole = require('./utils/initConsole');

var _initConsole2 = _interopRequireDefault(_initConsole);

var _helpers = require('./utils/helpers');

var _status = require('./status');

var _status2 = _interopRequireDefault(_status);

var _bootstrap = require('./bootstrap');

var _bootstrap2 = _interopRequireDefault(_bootstrap);

var _clean = require('./clean');

var _clean2 = _interopRequireDefault(_clean);

var _addRemoveUpgrade = require('./addRemoveUpgrade');

var _addRemoveUpgrade2 = _interopRequireDefault(_addRemoveUpgrade);

var _bump = require('./bump');

var _bump2 = _interopRequireDefault(_bump);

var _outdated = require('./outdated');

var _outdated2 = _interopRequireDefault(_outdated);

var _prepublish = require('./prepublish');

var _prepublish2 = _interopRequireDefault(_prepublish);

var _publish = require('./publish');

var _publish2 = _interopRequireDefault(_publish);

var _resetAllVersions = require('./resetAllVersions');

var _resetAllVersions2 = _interopRequireDefault(_resetAllVersions);

var _all = require('./all');

var _all2 = _interopRequireDefault(_all);

var _runScript = require('./runScript');

var _runScript2 = _interopRequireDefault(_runScript);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* eslint-disable max-len, global-require, import/no-dynamic-require, no-console */

process.env.YARN_SILENT = 0;

var pkg = require('../package.json');

var monorepoPkg = require(_path2.default.resolve('package.json'));

var OAO_CONFIG = monorepoPkg.oao || {};
var DEFAULT_SRC_DIR = OAO_CONFIG.src || 'packages/*';
var DEFAULT_COPY_ATTRS = 'description,keywords,author,license,homepage,bugs,repository';
var DEFAULT_CHANGELOG = 'CHANGELOG.md';

_commander2.default.version(pkg.version);

// =========================================
// Helpers
// =========================================
var processOptions = function processOptions(options0) {
  var options = options0;

  if (options.single) {
    options = (0, _timm.merge)(options, { src: [] });
  } else {
    // If workspaces are enabled in the monorepo, some configuration is
    // overriden by the monorepo package.json
    if (monorepoPkg.workspaces) {
      var src = monorepoPkg.workspaces;
      if ((0, _helpers.isObject)(src)) src = src.packages;
      if (!src) {
        throw new Error('Could not find correct config for Yarn workspaces');
      }
      options = (0, _timm.merge)(options, { src: src, workspaces: true });
    }

    // Add extra configuration in the `oao` field of the monorepo package.json
    options = (0, _timm.addDefaults)(options, { ignoreSrc: OAO_CONFIG.ignoreSrc });
  }

  return options;
};

// Create a command with common options
var createCommand = function createCommand(syntax, description) {
  return _commander2.default.command(syntax).description(description).option('-s --src <glob>', 'glob pattern for sub-package paths [' + DEFAULT_SRC_DIR + ']', DEFAULT_SRC_DIR).option('-i --ignore-src <glob>', 'glob pattern for sub-package paths that should be ignored').option('-l --link <regex>', 'regex pattern for dependencies that should be linked, not installed').option('--single', 'no subpackages, just the root one').option('--relative-time', 'shorten log dates');
};

// =========================================
// Commands
// =========================================
createCommand('status', 'Show an overview of the monorepo status').action(function (cmd) {
  var options = processOptions(cmd.opts());
  return (0, _status2.default)(options);
});

createCommand('bootstrap', 'Install external dependencies and create internal links').option('--prod --production', 'skip external and internal development-only dependencies (also via NODE_ENV=production)').option('--no-lockfile', "don't read or generate a lockfile").option('--pure-lockfile', "don't generate a lockfile").option('--frozen-lockfile', "don't generate a lockfile and fail if an update is needed").option('--no-parallel', "don't run yarn install in parallel (use it to debug errors, since parallel logs may be hard to read)").action(function (cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _bootstrap2.default)(options);
});

createCommand('clean', 'Delete all node_modules directories from sub-packages and the root package').action(function (cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _clean2.default)(options);
});

createCommand('add <sub-package> <packages...>', 'Add dependencies to a sub-package').option('-D --dev', 'add to `devDependencies` instead of `dependencies`').option('-P --peer', 'add to `peerDependencies` instead of `dependencies`').option('-O --optional', 'add to `optionalDependencies` instead of `dependencies`').option('-E --exact', 'install the exact version').option('-T --tilde', 'install the most recent release with the same minor version').action(function (subpackage, deps, cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _addRemoveUpgrade2.default)(subpackage, 'add', deps, options);
});

createCommand('remove <sub-package> <packages...>', 'Remove dependencies from a sub-package').action(function (subpackage, deps, cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _addRemoveUpgrade2.default)(subpackage, 'remove', deps, options);
});

createCommand('upgrade <sub-package> [packages...]', 'Upgrade some/all dependencies of a package').option('--ignore-engines', 'disregard engines check during upgrade').action(function (subpackage, deps, cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _addRemoveUpgrade2.default)(subpackage, 'upgrade', deps, options);
});

createCommand('bump <packages...>', 'Upgrade one or several dependencies throughout the monorepo (e.g. react@next, timm)').action(function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(deps, cmd) {
    var options;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            options = processOptions(cmd.opts());

            (0, _initConsole2.default)(options);
            _context.next = 4;
            return (0, _bump2.default)(deps, options);

          case 4:
            return _context.abrupt('return', (0, _bootstrap2.default)(options));

          case 5:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}());

createCommand('outdated', 'Check for outdated dependencies').action(function (cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _outdated2.default)(options);
});

createCommand('prepublish', 'Prepare for a release: validate versions, copy READMEs and package.json attrs').option('--copy-attrs <attrs>', 'copy these package.json attrs to sub-packages [' + DEFAULT_COPY_ATTRS + ']', DEFAULT_COPY_ATTRS).action(function (cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _prepublish2.default)(options);
});

createCommand('publish', 'Publish updated sub-packages').option('--no-master', 'allow publishing from a non-master branch').option('--no-check-uncommitted', 'skip uncommitted check').option('--no-check-unpulled', 'skip unpulled check').option('--no-checks', 'skip all pre-publish checks').option('--no-confirm', 'do not ask for confirmation before publishing').option('--no-git-commit', 'skip the commit-tag-push step before publishing').option('--no-npm-publish', 'skip the npm publish step').option('--new-version <version>', 'use this version for publishing, instead of asking').option('--increment-version-by <major|minor|patch|rc|beta|alpha>', 'increment version by this, instead of asking').option('--publish-tag <tag>', 'publish with a custom tag (instead of `latest`)').option('--changelog-path <path>', 'changelog path [' + DEFAULT_CHANGELOG + ']', DEFAULT_CHANGELOG).option('--no-changelog', 'skip changelog updates').action(function (cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _publish2.default)(options);
});

createCommand('reset-all-versions <version>', 'Reset all versions (incl. monorepo package) to the specified one').option('--no-confirm', 'do not ask for confirmation').action(function (version, cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _resetAllVersions2.default)(version, options);
});

createCommand('all <command>', 'Run a given command on all sub-packages').option('--tree', 'follow dependency tree (starting from the tree leaves)').option('--parallel', 'run command in parallel on all sub-packages').option('--no-parallel-logs', 'use chronological logging, even in parallel mode').option('--parallel-limit <#processes>', 'max number of processes to launch').option('--ignore-errors', 'do not stop even if there are errors in some packages').action(function (command, cmd) {
  // Extract arguments following the first separator (`--`) and
  // add them to the command to be executed
  var rawArgs = cmd.parent.rawArgs;

  var idxSeparator = rawArgs.indexOf('--');
  var finalCommand = idxSeparator >= 0 ? [command].concat(rawArgs.slice(idxSeparator + 1)).join(' ') : command;
  // Run the `all` command
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _all2.default)(finalCommand, options);
});

createCommand('run-script <command>', 'Run a given script on all sub-packages').option('--tree', 'follow dependency tree (starting from the tree leaves)').option('--parallel', 'run script in parallel on all sub-packages').option('--no-parallel-logs', 'use chronological logging, even in parallel mode').option('--parallel-limit <#processes>', 'max number of processes to launch').option('--ignore-errors', 'do not stop even if there are errors in some packages').action(function (command, cmd) {
  var options = processOptions(cmd.opts());
  (0, _initConsole2.default)(options);
  return (0, _runScript2.default)(command, options);
});

process.on('unhandledRejection', function (err) {
  console.error(err); // eslint-disable-line
  process.exit(1);
});
process.on('SIGINT', function () {
  process.exit(0);
});

// Syntax error -> show CLI help
_commander2.default.command('*', '', { noHelp: true }).action(function () {
  return _commander2.default.outputHelp();
});
if (process.argv.length <= 2) _commander2.default.outputHelp();

// Let's go!
_commander2.default.parse(process.argv);