'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _constants = require('./constants');

var calcGraph = function calcGraph(allSpecs) {
  var out = [];
  var pkgNames = Object.keys(allSpecs);
  if (!pkgNames.length) return out;

  // Build virtual root node
  var virtualRootDeps = {};
  pkgNames.forEach(function (name) {
    virtualRootDeps[name] = true;
  });
  var virtualRootNode = {
    name: '__VIRTUAL_ROOT__',
    specs: { dependencies: virtualRootDeps }
  };

  // Build graph starting from virtual root node, then remove it
  buildGraph(allSpecs, virtualRootNode, pkgNames, out);
  return out.slice(0, out.length - 1);
};

var buildGraph = function buildGraph(allSpecs, pkg, pkgNames, out) {
  var visited = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var name = pkg.name;

  visited.push(name);
  var internalDeps = getInternalDeps(pkg, pkgNames);
  for (var i = 0; i < internalDeps.length; i++) {
    var depName = internalDeps[i];
    if (visited.indexOf(depName) >= 0) continue;
    buildGraph(allSpecs, allSpecs[depName], pkgNames, out, visited);
  }
  out.push(name);
};

var getInternalDeps = function getInternalDeps(pkg, pkgNames) {
  var specs = pkg.specs;

  var internalDeps = {};
  for (var i = 0; i < _constants.DEP_TYPES.length; i++) {
    var depType = _constants.DEP_TYPES[i];
    var deps = specs[depType] || {};
    var depNames = Object.keys(deps);
    for (var k = 0; k < depNames.length; k++) {
      var pkgName = depNames[k];
      if (pkgNames.indexOf(pkgName) >= 0) internalDeps[pkgName] = true;
    }
  }
  return Object.keys(internalDeps);
};

exports.default = calcGraph;