'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseDep = exports.dependsOn = exports.delay = exports.isObject = exports.shortenName = undefined;

var _constants = require('./constants');

var shortenName = function shortenName(name, maxLen) {
  if (name.length <= maxLen) return name;
  return name.slice(0, 2) + '\u2026' + name.slice(-(maxLen - 3));
};

var isObject = function isObject(o) {
  return !!o && o.constructor === Object;
};

var delay = function delay(ms) {
  return new Promise(function (resolve) {
    setTimeout(resolve, ms);
  });
};

var dependsOn = function dependsOn(pkg, possibleDep) {
  var specs = pkg.specs;

  for (var i = 0; i < _constants.DEP_TYPES.length; i++) {
    var depType = _constants.DEP_TYPES[i];
    var deps = specs[depType] || {};
    if (deps[possibleDep]) return true;
  }
  return false;
};

var parseDep = function parseDep(dep) {
  // Extract package name from the dependency specs
  // (forget about the first character, for compatibility with scoped packages)
  var idx = dep.indexOf('@', 1);
  var name = idx >= 1 ? dep.slice(0, idx) : dep;
  var version = idx >= 1 ? dep.slice(idx + 1) : '';
  return { name: name, version: version };
};

exports.shortenName = shortenName;
exports.isObject = isObject;
exports.delay = delay;
exports.dependsOn = dependsOn;
exports.parseDep = parseDep;