'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _storyboard = require('storyboard');

var _storyboardListenerConsoleParallel = require('storyboard-listener-console-parallel');

var _storyboardListenerConsoleParallel2 = _interopRequireDefault(_storyboardListenerConsoleParallel);

var _readSpecs = require('./readSpecs');

var _shell = require('./shell');

var _helpers = require('./helpers');

var _calcGraph = require('./calcGraph');

var _calcGraph2 = _interopRequireDefault(_calcGraph);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* eslint-disable no-constant-condition */

var DELAY_MAIN_LOOP = 20; // [ms]
var PLACEHOLDER_COMMAND = '__OAO_PLACEHOLDER_COMMAND__';

// ------------------------------------------------
// Main
// ------------------------------------------------
var multiRun = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref2, getCommandsForSubpackage) {
    var src = _ref2.src,
        ignoreSrc = _ref2.ignoreSrc,
        useTree = _ref2.tree,
        parallel = _ref2.parallel,
        parallelLogs = _ref2.parallelLogs,
        parallelLimit = _ref2.parallelLimit,
        ignoreErrors = _ref2.ignoreErrors,
        relativeTime = _ref2.relativeTime;

    var allJobs, allSpecs, pkgNames, _loop, i;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (parallel && parallelLogs) {
              (0, _storyboard.removeAllListeners)();
              (0, _storyboard.addListener)(_storyboardListenerConsoleParallel2.default, { relativeTime: relativeTime });
            }

            // Gather all jobs
            allJobs = [];
            _context.next = 4;
            return (0, _readSpecs.readAllSpecs)(src, ignoreSrc, false);

          case 4:
            allSpecs = _context.sent;
            pkgNames = useTree ? (0, _calcGraph2.default)(allSpecs) : Object.keys(allSpecs);

            _loop = function _loop(i) {
              var pkgName = pkgNames[i];
              var pkg = allSpecs[pkgName];
              var pkgPath = pkg.pkgPath;

              var storySrc = parallel && !parallelLogs ? (0, _helpers.shortenName)(pkgName, 20) : undefined;
              var commands = getCommandsForSubpackage(pkg.specs);
              if (commands.length) {
                commands.forEach(function (cmd) {
                  allJobs.push({
                    cmd: cmd,
                    cwd: pkgPath,
                    bareLogs: !!parallelLogs,
                    storySrc: storySrc,
                    status: 'idle',
                    pkg: pkg
                  });
                });
              } else if (useTree) {
                // Suppose A --> B --> C (where --> means "depends on"),
                // and B generates no jobs, whilst A and C do.
                // Creating a placeholder job for B simplifies getNextJob(),
                // since it will only need to check direct dependencies between
                // subpackages
                allJobs.push({
                  cmd: PLACEHOLDER_COMMAND,
                  cwd: pkgPath,
                  bareLogs: false,
                  status: 'idle',
                  pkg: pkg
                });
              }
            };

            for (i = 0; i < pkgNames.length; i += 1) {
              _loop(i);
            }

            // Run in serial or parallel mode

            if (parallel) {
              _context.next = 13;
              break;
            }

            _context.next = 11;
            return runSerially(allJobs, { ignoreErrors: ignoreErrors });

          case 11:
            _context.next = 15;
            break;

          case 13:
            _context.next = 15;
            return runInParallel(allJobs, {
              ignoreErrors: ignoreErrors,
              parallelLogs: parallelLogs,
              parallelLimit: parallelLimit,
              useTree: useTree
            });

          case 15:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function multiRun(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

// ------------------------------------------------
// Serial and parallel runners
// ------------------------------------------------
var runSerially = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(allJobs, _ref4) {
    var ignoreErrors = _ref4.ignoreErrors;
    var i, job;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            i = 0;

          case 1:
            if (!(i < allJobs.length)) {
              _context2.next = 9;
              break;
            }

            job = allJobs[i];

            executeJob(job, { ignoreErrors: ignoreErrors });
            _context2.next = 6;
            return job.promise;

          case 6:
            i++;
            _context2.next = 1;
            break;

          case 9:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function runSerially(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var runInParallel = function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(allJobs, _ref6) {
    var ignoreErrors = _ref6.ignoreErrors,
        parallelLogs = _ref6.parallelLogs,
        parallelLimit = _ref6.parallelLimit,
        useTree = _ref6.useTree;
    var maxConcurrency, job, pendingPromises;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            maxConcurrency = parallelLimit || Infinity;

          case 1:
            if (!true) {
              _context3.next = 17;
              break;
            }

            if (!(getIdleJobs(allJobs).length === 0)) {
              _context3.next = 4;
              break;
            }

            return _context3.abrupt('break', 17);

          case 4:

            // Get a job!
            job = getNextJob(allJobs, { useTree: useTree });

            if (!job) {
              _context3.next = 13;
              break;
            }

            if (!(getRunningJobs(allJobs).length >= maxConcurrency)) {
              _context3.next = 10;
              break;
            }

            _context3.next = 9;
            return (0, _helpers.delay)(DELAY_MAIN_LOOP);

          case 9:
            return _context3.abrupt('continue', 1);

          case 10:
            executeJob(job, { ignoreErrors: ignoreErrors });
            _context3.next = 15;
            break;

          case 13:
            _context3.next = 15;
            return (0, _helpers.delay)(DELAY_MAIN_LOOP);

          case 15:
            _context3.next = 1;
            break;

          case 17:
            if (!parallelLogs) {
              _context3.next = 34;
              break;
            }

            pendingPromises = allJobs.filter(function (o) {
              return o.status !== 'done';
            }).map(function (job) {
              return job.promise;
            });
            _context3.prev = 19;
            _context3.next = 22;
            return Promise.all(pendingPromises);

          case 22:
            _context3.next = 33;
            break;

          case 24:
            _context3.prev = 24;
            _context3.t0 = _context3['catch'](19);

            if (!_context3.t0.stderr) {
              _context3.next = 32;
              break;
            }

            console.error(_context3.t0.message); // eslint-disable-line
            console.error(_context3.t0.stderr); // eslint-disable-line
            throw new Error(_context3.t0.message);

          case 32:
            throw _context3.t0;

          case 33:
            process.exit(0);

          case 34:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined, [[19, 24]]);
  }));

  return function runInParallel(_x5, _x6) {
    return _ref5.apply(this, arguments);
  };
}();

// ------------------------------------------------
// Helpers
// ------------------------------------------------
/* eslint-disable no-param-reassign */
var executeJob = function executeJob(job, _ref7) {
  var ignoreErrors = _ref7.ignoreErrors;

  job.promise = _executeJob(job, { ignoreErrors: ignoreErrors });
};

var _executeJob = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(job, _ref9) {
    var ignoreErrors = _ref9.ignoreErrors;
    var cmd, cwd, bareLogs, storySrc, promise;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            cmd = job.cmd, cwd = job.cwd, bareLogs = job.bareLogs, storySrc = job.storySrc;

            if (!(cmd === PLACEHOLDER_COMMAND)) {
              _context4.next = 4;
              break;
            }

            job.status = 'done';
            return _context4.abrupt('return');

          case 4:
            promise = (0, _shell.exec)(cmd, { cwd: cwd, bareLogs: bareLogs, storySrc: storySrc });

            job.status = 'running';
            _context4.prev = 6;
            _context4.next = 9;
            return promise;

          case 9:
            job.status = 'done';
            _context4.next = 17;
            break;

          case 12:
            _context4.prev = 12;
            _context4.t0 = _context4['catch'](6);

            job.status = 'done';

            if (ignoreErrors) {
              _context4.next = 17;
              break;
            }

            throw _context4.t0;

          case 17:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined, [[6, 12]]);
  }));

  return function _executeJob(_x7, _x8) {
    return _ref8.apply(this, arguments);
  };
}();
/* eslint-enable no-param-reassign */

var getNextJob = function getNextJob(jobs, _ref10) {
  var useTree = _ref10.useTree;

  for (var i = 0; i < jobs.length; i++) {
    var candidateJob = jobs[i];
    if (candidateJob.status !== 'idle') continue;
    var candidateJobPkg = candidateJob.pkg;

    var isFound = true;
    if (useTree) {
      // Check whether a previous job that hasn't finished
      // belongs to a direct dependency of the candidate (notice
      // that we have _placeholder_ jobs, so we don't need to worry
      // about packages that are indirect dependencies.
      for (var k = 0; k < i; k++) {
        var previousJob = jobs[k];
        if (previousJob.status === 'done') continue;
        var previousJobPkg = previousJob.pkg;

        if ((0, _helpers.dependsOn)(candidateJobPkg, previousJobPkg.name)) {
          isFound = false;
          break;
        }
      }
    }
    if (isFound) return candidateJob;
  }
  return null;
};
var getRunningJobs = function getRunningJobs(jobs) {
  return jobs.filter(function (job) {
    return job.status === 'running';
  });
};
var getIdleJobs = function getIdleJobs(jobs) {
  return jobs.filter(function (job) {
    return job.status === 'idle';
  });
};

// ------------------------------------------------
// Public
// ------------------------------------------------
exports.default = multiRun;